/**************************************************************************************************
Copyright (c) 2005-2006 Darrell Tam
email: darrell.barrell@gmail.com

Top level DtBlkFx GUI class
please see Gui.cpp for more information

History
	Date       Version    Programmer         Comments
	16/2/03    1.0        Darrell Tam		Created


This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

***************************************************************************************************/

#ifndef __DT_BLKFX_Gui__
#define __DT_BLKFX_Gui__

#include <dt/misc_stuff.h>
#include <dt/VstGuiSupport.h>
#include <dt/cplx.h>

#include "PixelFreqMap.h"
#include "GradColMap.h"
#include "BlkFxParam.h"
#include "Spectrogram.h"
#include "FxCtrl.h"
#include "GlobalCtrl.h"

//-----------------------------------------------------------------------------
class Gui : public AEffGUIEditor
{
public:
	Gui (AudioEffect *effect);
	virtual ~Gui ();

	DtBlkFx* blkFx() { return (DtBlkFx*)effect; }

public: // methods called by DtBlkFx class
	void FFTDataRdy(
		int a // 0=input data, 1=output data
	);

	void suspend ();
	void resume ();
	bool keysRequired ();

public:  // override 

	virtual long open (void *ptr);
	virtual void idle ();
	virtual void setParameter (long index, float value);
	virtual void close ();
	
	// VST 2.1
	virtual long onKeyDown (VstKeyCode &keyCode);
	virtual long onKeyUp (VstKeyCode &keyCode);

public: // internal stuff
	void update(CDrawContext *context);

	void generateAmpOverlay();

	// gradient colour map to draw spectrum with
	GradColMap		_col_map;

	// pixel to frequency mapping
	PixelFreqMap	_pix_freq;

	// pixel to bin mappings
	PixelFreqMap::BinRng _pix_bin[BlkFxParam::AUDIO_CHANNELS];

	VstGuiNoRefCount<GlobalCtrl> _glob_ctrl;

	// spectrograms for input & output (for mono) or various in stereo
	VstGuiNoRefCount<Spectrogram> _sgram[2];

	// fx controls
	VstGuiNoRefCount<FxCtrl> _fx_ctrl[16];

	// gui elements used by FxCtrl
	// amplify overlay graphics
	VstGuiRef<CAlphaContext> _amp_bm;

	//VstGuiRef<CBitmap> _freq_bg_bm;
	VstGuiRef<CAlphaContext> _freq_bg_bm;
	
	VstGuiRef<CAlphaContext> _freq_knob_bm;

	VstGuiRef<CAlphaContext> _fxval_knob_bm;

	// true if gui is open ok
	bool _ok;
};

#endif
